/* FFT.java */
/* from Java$B$K$h$k%"%k%4%j%:%`<-E5(B */

public class FFT {
    int n;
    int[] bitrev;
    double[] sintbl;

    public FFT(int n) {
	this.n = n;
	sintbl = new double[n+n/4];
	bitrev = new int[n];

	double t = Math.sin(Math.PI/n);
	double dc = 2*t*t;
	double ds = Math.sqrt(dc*(2-dc));
	t = 2*dc;
	double c = sintbl[n/4] = 1;
	double s = sintbl[0] = 0;
	for (int i = 1; i < n/8; i++) {
	    c -= dc;
	    dc += t*c;
	    s += ds;
	    ds -= t*s;
	    sintbl[i] = s;
	    sintbl[n/4-i] = c;
	}
	if (n/8 != 0)
	    sintbl[n/8] = Math.sqrt(0.5);
	for (int i = 0; i < n/4; i++)
	    sintbl[n/2-i] = sintbl[i];
	for (int i = 0; i < n/2+n/4; i++)
	    sintbl[i+n/2] = -sintbl[i];

	int i = 0;
	int j = 0;
	int k;
	bitrev[0] = 0;
	while (++i < n) {
	    k = n/2;
	    while (k <= j) {
		j -= k;
		k /= 2;
	    }
	    j += k;
	    bitrev[i] = j;
	}
    }

    public void fftsub(double[] x, double[] y, int sign) {
	int i, j;
	double t;
	for (i = 0; i < n; i++) {
	    j = bitrev[i];
	    if (i < j) {
		t = x[i];
		x[i] = x[j];
		x[j] = t;
		t = y[i];
		y[i] = y[j];
		y[j] = t;
	    }
	}
	int h, d, ik;
	double c, s, dx, dy;
	for (int k = 1; k < n; k *= 2) {
	    h = 0;
	    d = n/(k*2);
	    for (j = 0; j < k; j++) {
		c = sintbl[h+n/4];
		s = sign*sintbl[h];
		for (i = j; i < n; i += k*2) {
		    ik = i+k;
		    dx = s*y[ik]+c*x[ik];
		    dy = c*y[ik]-s*x[ik];
		    x[ik] = x[i]-dx;
		    x[i] += dx;
		    y[ik] = y[i]-dy;
		    y[i] += dy;
		}
		h += d;
	    }
	}
    }

    public void fft(double[] x, double[] y) {
	fftsub(x, y, 1);
    }

    public void ifft(double[] x, double[] y) {
	fftsub(x, y, -1);
	for (int i = 0; i < n; i++) {
	    x[i] /= n;
	    y[i] /= n;
	}
    }
}